#include "q3la.h"

q3la_main::q3la_main()
{
}

q3la_main::~q3la_main()
{
}

void q3la_main::run (int argc, char** argv) 
{
#ifdef TEST
	_chdir ("c:\\Games\\Quake III Arena\\baseq3");
//	_chdir ("c:\\");
//	config.no_bots = true;
//	config.limits_used = true;
	argc = 2;
	argv[1] = "games6.log";
#endif

	if (argc < 2) {
		help();
	}

	version_info();

	// Get start time

	clock_t start, finish;   
	start = clock();

	config.read_options (argc, argv); // read command line options - needed for read_cfg
	printf ("\nReading CFG: %s", config.configfile);
	config.read_cfg (config.configfile); // read CFG
	printf ("\n");
	config.read_options (argc, argv); // read command line options again to override CFG

	if (config.q3ladata == true) { // delete q3ladata.txt
		outputfile deldata; 
		deldata.open("q3ladata.txt");
		deldata.close();
	}

	analyze (argv[1]);

	// Get finish time, output
	
	finish = clock();
	printf( "\n%.2f seconds elapsed", ((double) (finish - start) / CLOCKS_PER_SEC));

	if (config.open_after == true) { // NT / 9x check!!!
		system ("start players.htm"); //9x
		system ("players.htm"); // NT
	}
}

void q3la_main::analyze (char* pathstr) 
{
/*
	struct tm *newtime;
	time_t long_time;

	time( &long_time );                
	newtime = localtime( &long_time );
*/

	printf ("\nParsing %s\n", pathstr);

	FILE* logfile = fopen (pathstr, "rb");
	if (logfile == 0) {
		error::msg_and_exit ("q3la_main::analyze - can't open %s", pathstr);
	}

	fseek (logfile, 0, SEEK_END);
	int logfilesize = ftell (logfile);
	fseek (logfile, 0, SEEK_SET);

	// Backup start directory

	char start_directory[256];
#ifndef __LINUX_PLATFORM__
	_getcwd (start_directory, 256);
#else
	getcwd (start_directory, 256);
#endif	

	// Change to output directory

	if (config.output_directory[0] != 0) {
		printf ("\nChanging to directory \"%s\"\n", config.output_directory);
	}

	if (_chdir (config.output_directory) == -1) {
#ifndef __LINUX_PLATFORM__
		_mkdir (config.output_directory);
#else
		mkdir (config.output_directory, S_IRWXU);
#endif
		_chdir (config.output_directory);
	}

	// Delete directory "q3la_games"

	diverse::deltree ("q3la_games");
	diverse::deltree ("q3la_players");

	// Write stats

	printf ("\nWriting game stats to \"games.htm\"        ");

	outputfile out_games_total;
	out_games_total.open ("games.htm");
	html::html_header (&out_games_total);
	if (config.homepage_address[0] != 0) {
		html::line (&out_games_total, "<a href=\"%s\"><font color=\"#FF0000\" size=\"4\"><b>%s</b></font></a>", 
			config.homepage_address, config.homepage_string);
	}
	html::line (&out_games_total, "<a href=\"players.htm\"><font color=\"#FF0000\" size=\"4\"><b>Player list</b></font></a>");
	html::line (&out_games_total, "Games with less than two players are not listed\r\n<br>Players without kills or deaths are ignored");

	resultdata_total results_total;
	int	gamecount = 0;

	resultdata result(&config);
	analyzedemo analyze (logfile, &result, &config);
	for (;;) {
		result.init();
		if (analyze.parse() == 0) { // parse next game
			break;
		}
		result.sort_players(so_skill); // sort players for each game
		gamecount++;

		int rel_pos = 100 * ftell(logfile) / logfilesize;
		printf ("\b\b\b\b\b\b\b(%3i %%)", rel_pos); // progress indication

		if (output_game(gamecount, &result, &out_games_total) == 0) {
			results_total.add_results (&(result)); // add to total results 
		}
	}
	printf ("\b\b\b\b\b\b\b(100 %%)"); // progress indication

	
	html::html_footer(&out_games_total, true);
	out_games_total.close();

	if (config.q3ladata == true) { 
		printf ("\nWriting q3ladata.txt"); // output of this is delayed to actual writing
	}

	// Output total player results

	outputfile outfile;
	outfile.open ("players.htm");
	printf ("\nWriting player stats to \"players.htm\"");

	results_total.sort_players (config.sortorder_players); // sort all players
	output out_total (&results_total, &outfile, &config, gamecount);
	out_total.list_players_total_html();

	outfile.close ();

	// Output per player results

#ifndef __LINUX_PLATFORM__
	_mkdir ("q3la_players");
#else
	mkdir ("q3la_players", S_IRWXU);
#endif
	_chdir ("q3la_players");

	printf ("\nWriting per player stats        ");
	output_per_player_stats(&results_total);
	printf ("\n");

	// Close logfile
	
	fclose (logfile);

	if (config.output_directory[0] != 0) {
		printf ("\nChanging back to directory \"%s\"\n", start_directory);
	}

	_chdir (start_directory); // go back to start directory
}

void q3la_main::output_per_player_stats(resultdata_total* results_total)
{
	player_t* player = results_total->get_player_first()->next;
	int count = 0;
	while (player != 0) {
		count++;
		if (player->listed == false) {
			player = player->next;
			continue;
		}
		int rel_pos = 100 * count / results_total->get_player_count();
		printf ("\b\b\b\b\b\b\b(%3i %%)", rel_pos); // progress indication
		outputfile out_player;
		char filename[256];
		sprintf (filename, "player_%i.htm", count);
		out_player.open (filename);
		html::html_header (&out_player);
		html::line (&out_player, "<a href=\"../players.htm\"><font color=\"#FF0000\" size=\"4\"><b>Player list</b></font></a>");
		html::line (&out_player, "<font size=4><b><span style=\"background-color: #808080\">%s</span></b></font>", 
			diverse::funname_to_html(player->get_name()));
		output out (results_total, &out_player, &config, 0);
		out.player_weapon_stats(player);
		html::html_footer (&out_player, false);
		out_player.close();
		player = player->next;
	}
	printf ("\b\b\b\b\b\b\b(100 %%)"); // progress indication
}

int q3la_main::output_game(int gamecount, resultdata* result, outputfile* out_games_total)
{
	// delete bots
	if (config.no_bots == true) {
		result->delete_bots();
	}

	// delete nullplayers
	result->delete_nullplayers();

	// only output games with more than one player
	if (result->get_player_count() < 2) {
		return 1;
	}

	// Output to "games.htm"
	output out_total (result, out_games_total, &config);
	html::line_header(out_games_total);
	out_games_total->write ("<b><font size=\"4\"><a href=\"q3la_games/game_%i.htm\"><font color=\"#FF0000\">Game %i</font></a><br>(%s)</font></b>", gamecount, gamecount, result->mapname);
	html::line_footer(out_games_total);
	out_total.list_players_html();
	
	// Open output file for current game

#ifndef __LINUX_PLATFORM__
	_mkdir ("q3la_games");
#else
	mkdir ("q3la_games", S_IRWXU);
#endif
	_chdir ("q3la_games");

	char gamefile[16];
	sprintf (gamefile, "game_%i.htm", gamecount);
	outputfile out_game_current;
	out_game_current.open (gamefile);
	html::html_header(&out_game_current);
	
	// Output to "game_<x>.htm"
	output out_current (result, &(out_game_current), &config);
	if (config.homepage_address[0] != 0) {
		html::line (&out_game_current, "<a href=\"%s\"><font color=\"#FF0000\" size=\"4\"><b>%s</b></font></a>", 
			config.homepage_address, config.homepage_string);
	}
	html::line (&out_game_current, "<a href=\"../games.htm\"><font color=\"#FF0000\" size=\"4\"><b>Game list</b></font></a>");
	html::line (&out_game_current, "<b><font color=\"#FF0000\" size=\"4\">Game %i</font></b>", gamecount);
	out_current.game_info();
	out_current.list_players_html();
	html::line (&out_game_current, "<b>Killcharts</b>");
	for (int i = 0; i < MAX_WEAPONS; i++) {
		out_current.table_kills_html(i);
	}
	if (config.q3ladata == true) {
		out_current.game_data(gamecount);
	}
	
	// Close current game file
	html::html_footer(&out_game_current, true);
	out_game_current.close();
	_chdir ("..");

	return 0;
}

void q3la_main::help () 
{
	version_info();
    printf ("\nUsage: q3la <logfile> [OPTIONS]\n");
    printf ("\nOptions:\n");
	printf ("\n/a                 open results after analysis");
	printf ("\n/b                 don't include bots in results");
	printf ("\n/d                 write data file (q3ladata.txt)");
	printf ("\n/o<x>              sortorder (<x> can be N,R,E,S,F,K,N,P - see readme)");
	printf ("\n/cfg:<filespec>    use custom CFG - no spaces in <filespec> allowed");
	printf ("\n/min_kills:<x>     exclude players with less than <x> kills from player stats");
	printf ("\n/min_games:<x>     exclude players with less than <x> games from player stats");
	printf ("\n/min_playtime:<x>  exclude players with less than <x> seconds playtime");
	printf ("\n/max_players:<x>   no more than <x> players in player stats");
	printf ("\n\nOptions can be specified in upper or lower case with the prefix '/' or '-'");
	printf ("\n");
	exit (1);
}

void q3la_main::version_info()
{
    printf ("\n%s %s, %s\n%s\n", helpstr1, version, date, helpstr2);
	printf ("\n*** ONLY WORKS WITH Q3A'S GAMES.LOG! ***\n");
}
